#<profile4xi profile = "11" />  
# -------------------------------------------------------------------------------------------------
#<program4xi program = "zap3000six4xi.py">
#<abstract4xi abstract = "zaps to direction six"/>
#<verify4xi verification = "11" target = "zap2actor4xi.py"/>
#<refer4xi reference = "zap2actor4xi.py"/>
#<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'zap3000six4xi']
#"
#/>

# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
import itertools
import logging 
import os
import threading                
import signal
import sys
from math import pi
from math import sin
from math import cos 
from math import fabs
import weakref
import _thread
import _winapi
 
#"
#/>

# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from zap2module4xi import ModuleKB2Zap4Xi
from print4xi      import print4xi
from time4xi       import time4xi
#"
#/>

# -------------------------------------------------------------------------------------------------
# Global constants                            
# ------------------------------------------------------------------------------------------------
#<globalize4xi globalization = "
# do NO define any globals for this kind of multiprocessing/thread module !!
#"
#/>

# -------------------------------------------------------------------------------------------------
# Settings
# -------------------------------------------------------------------------------------------------
#<set4xi setting = "
ack2debug = False
ack2dragontrace = False 
#" 
#"/>


# -------------------------------------------------------------------------------------------------
#<function4xi function = "zap3000six4xi" usage = "target function for zap0000six4xi with Process4Xi">
#<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting"/> 
# -------------------------------------------------------------------------------------------------
def zap3000six4xi(queue3000six, consumer_channel3000six, publisher_channel3000six, kb3000six):  
    '''
    returns layer 3000 of direction six
    ''' 
    # -------------------------------------------------------------------------------------------------
    # (Step 1/33)             
    # -------------------------------------------------------------------------------------------------   
    class2print  = 'zap3000six4xi'
    method2print = 'zap3000six4xi'
        
    # -------------------------------------------------------------------------------------------------
    # (Step 2/33)                      
    # -------------------------------------------------------------------------------------------------                                                               
    if ack2debug is True:
        print("\n (Step 2A/33) OF ZAP3000SIX4XI - STARTED TIME: ", time4xi())                                                       #  
        print("\n (Step 2B/33) OF ZAP3000SIX4XI -  QUEUE3000SIX: ", queue3000six)                                                   # 
        print("\n (Step 2C/33) OF ZAP3000SIX4XI -  CONSUME_CHANNEL3000SIX: ", consumer_channel3000six)                              # 
        print("\n (Step 2D/33) OF ZAP3000SIX4XI -  PUBLISHER_CHANNEL3000SIX: ", publisher_channel3000six)                           # 
        print("\n (Step 2E/33) OF ZAP3000SIX4XI -  KB3000SIX: ", kb3000six)                                                         
        # 
        
    message2print = "{(Step 2A/33) started" + "; zap3000six4xi-started: " + str(zap3000six4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                                              # 
        
    message2print = "{(Step 2B/33) passed" + "; queue3000six-passed-to-zap3000six4xi: " + str(queue3000six) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                                              # 
                  
    message2print = "{(Step 2C/33) passed" + "; consumer-channel3000six-passed-to-zap3000six4xi: " + str(consumer_channel3000six) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                                              #    
    
    message2print = "{(Step 2D/33) passed" + "; publisher-channel3000six-passed-to-zap3000six4xi: " + str(publisher_channel3000six) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                                              #          
    
    message2print = "{(Step 2E/33) passed" + "; kb3000six-passed-to-zap3000six4xi: " + str(kb3000six) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                                              #          
    
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    # (Step 3/33)
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    if ack2debug is True:
        print("\n (Step 3/33) OF ZAP3000SIX4XI - TRYING TO PUT LAYER 3000 TO QUEUE3000SIX: ", queue3000six)                         # 
    message2print = "{(Step 3/33) using" + "; queue3000six-using-by-zap3000six4xi: " + str(queue3000six)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                                              #  
            
    ack3000put4six = False                                                                                                                           

    # ------------------------------------------------------------------------------------- 
    # (Step 4/33) 
    # -------------------------------------------------------------------------------------                 

    # ------------------------------------------------------------------------------------- 
    # CONSTANTS VALID FOR ANY ZAP-KOWLEDGEBASE  
    # -------------------------------------------------------------------------------------     
    key2sense       = '2xi'
    key2reason      = '4xi'
    key2ace         = '8xi'
    key2direction   = 1
    key2distance    = 2
    key2layer       = 3
    
    key2area4layer  = 1
    key2color4layer = 2
    key2qty4layer   = 3
    key2area4node   = 4
    
    key2identity    = 4 
    key2color       = 5
    key2fire        = 6
    key2cluster     = 7
    key2ray         = 8
    key2deviation   = 9 
    key2super       = 10
    key2message     = 11
    key2actor       = 12
    key2type        = 13
    key2home        = 14
    key2previous    = 15
    key2next        = 16
    key2last        = 17
    key2first       = 18
    key2module      = 19
      
    # ------------------------------------------------------------------------------------- 
    # (Step 5/33) CONSTANTS VALID FOR LAYER 3000 OF DIRECTION SIX 
    # ------------------------------------------------------------------------------------- 
    direction3000sense4six = 6                                                                                                      # DIRECTION
    message2print = "{(Step 5A/33) set" + "; direction3000sense4six-set-by-zap3000six4xi to: " + str(direction3000sense4six)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                                              #  
            
    distance3000sense4six  = 3000                                                                                                   # DISTANCE = LAYER
    message2print = "{(Step 5B/33) set" + "; distance3000sense4six-set-by-zap3000six4xi to: " + str(distance3000sense4six)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                                              #  
                                                                 
    # ------------------------------------------------------------------------------------- 
    # (Step 6/33) CREATE NEW KNOWLEDGEBASE FOR LAYER 3000 OF DIRECTION SIX 
    # -------------------------------------------------------------------------------------         
    keys3000six = kb3000six.keys()
    kb3000six2put = {}
    for key3000six in keys3000six:
        if ((kb3000six[key3000six] [key2sense] [key2distance] == distance3000sense4six) and (kb3000six[key3000six] [key2sense] [key2direction] == direction3000sense4six)):
            # ---------------------------------------------------------------------------------------------------------------------- 
            # 
            # ----------------------------------------------------------------------------------------------------------------------        
            if ack2debug is True:
                print("\n (Step 6A0/33) OF ZAP3000SIX4XI - LEVEL NODE'S KEY: ", key3000six, " CONTENT: ", kb3000six[key3000six])
            #  
            message2print = "{(Step 6A0/33) set" + "; nodekey-set-by-zap3000six4xi to: " + str(key3000six)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                                      #                    
            # ---------------------------------------------------------------------------------------------------------------------- 
            # 
            # ----------------------------------------------------------------------------------------------------------------------         
            try:                                                                                                                    
                kb3000six2put [key3000six] = {}                                                                                     # SAVE THE LAYER OF THE REQUIRED DIRECTION
            except:
                print("\n (Step 6A/33) OF ZAP3000SIX4XI *** FAILED TO SET LAYER 3000 TO QUEUE3000SIX: ", key3000six)
            # ---------------------------------------------------------------------------------------------------------------------- 
            # 
            # ----------------------------------------------------------------------------------------------------------------------                 
            try:         
                kb3000six2put [key3000six] [key2sense]  = {}                                                                        # SENSE
            except:
                print("\n (Step 6B/33) OF ZAP3000SIX4XI *** FAILED TO SET LAYER 3000 TO QUEUE3000SIX: ", key3000six)
            # ---------------------------------------------------------------------------------------------------------------------- 
            # 
            # ----------------------------------------------------------------------------------------------------------------------                   
            if ack2debug is True:
                print("\n (Step 6C/33) OF ZAP3000SIX4XI - TRYING TO SET DIRECTION TO : ", kb3000six [key3000six] [key2sense] )      # 
            try:                             
                kb3000six2put [key3000six] [key2sense] [key2direction] = kb3000six [key3000six] [key2sense] [key2direction]         # DIRECTION 
                message2print = "{(Step 6D/33) set" + "; direction-of-zap-set-by-zap3000six4xi to: " + str(kb3000six2put [key3000six] [key2sense] [key2direction])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                                  # 
                         
            except:
                print("\n (Step 6E/33) OF ZAP3000SIX4XI *** FAILED TO SET LAYER 3000 TO QUEUE3000SIX: ", key3000six)
            # ---------------------------------------------------------------------------------------------------------------------- 
            # 
            # ----------------------------------------------------------------------------------------------------------------------                 
            if ack2debug is True:
                print("\n (Step 6F/33) OF ZAP3000SIX4XI - TRYING TO SET DISTANCE TO: ", kb3000six [key3000six] [key2sense] )      #                
            try:                             
                kb3000six2put [key3000six] [key2sense] [key2distance] = kb3000six [key3000six][key2sense] [key2distance]            # DISTANCE
                message2print = "{(Step 6G/33) set" + "; distance-of-zap-set-by-zap3000six4xi to: " + str(kb3000six2put [key3000six] [key2sense] [key2distance])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                                  # 
            except:
                print("\n (Step 6B4B/33) OF ZAP3000SIX4XI *** FAILED TO SET LAYER 3000 TO QUEUE3000SIX: ", key3000six)
            # ---------------------------------------------------------------------------------------------------------------------- 
            # 
            # ----------------------------------------------------------------------------------------------------------------------                 
            if ack2debug is True:
                print("\n (Step 6B5A/33) OF ZAP3000SIX4XI - TRYING TO SET LAYER TO: ", kb3000six [key3000six] [key2sense] )       #        
            try:                             
                kb3000six2put [key3000six] [key2sense] [key2layer] = {}                                                             # LAYER
                message2print = "{(Step 6B4A/33) set" + "; layer-of-zap-set-by-zap3000six4xi to: " + str(kb3000six2put [key3000six] [key2sense] [key2layer])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                                  #                 
            except:
                print("\n (Step 6B5B/33) OF ZAP3000SIX4XI *** FAILED TO SET LAYER 3000 TO QUEUE3000SIX: ", key3000six)
            # ---------------------------------------------------------------------------------------------------------------------- 
            # 
            # ----------------------------------------------------------------------------------------------------------------------                 
            if ack2debug is True:
                print("\n (Step 6B6A/33) OF ZAP3000SIX4XI - TRYING TO SET AREA FOR LAYER TO: ", kb3000six [key3000six]  [key2sense] [key2layer] )                   
            try:                                                                                                                    #                     
                kb3000six2put [key3000six] [key2sense] [key2layer] [key2area4layer] = kb3000six [key3000six] [key2sense] [key2layer] [key2area4layer] 
                                                                                                                                    # AREA2LAYER
                message2print = "{(Step 6B4A/33) set" + "; area2layer-of-zap-set-by-zap3000six4xi to: " + str(kb3000six2put [key3000six] [key2sense] [key2layer] [key2area4layer])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                                  #               
            except:
                print("\n (Step 6B6B/33) OF ZAP3000SIX4XI *** FAILED TO SET LAYER 3000 TO QUEUE3000SIX: ", key3000six)
            # ---------------------------------------------------------------------------------------------------------------------- 
            # 
            # ----------------------------------------------------------------------------------------------------------------------                 
            if ack2debug is True:
                print("\n (Step 6B7A/33) OF ZAP3000SIX4XI - TRYING TO SET COLOR TO LAYER TO: ", kb3000six [key3000six]  [key2sense] [key2layer] )
                                                                                                                                    #                    
            try:                             
                kb3000six2put [key3000six] [key2sense] [key2layer] [key2color4layer] = kb3000six [key3000six] [key2sense] [key2layer] [key2color4layer]
                                                                                                                                    # 
                message2print = "{(Step 6B4A/33) set" + "; color2layer-of-zap-set-by-zap3000six4xi to: " + str(kb3000six2put [key3000six] [key2sense] [key2layer] [key2color4layer])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                                  #             
            except:
                print("\n (Step 6B7B/33) OF ZAP3000SIX4XI *** FAILED TO SET LAYER 3000 TO QUEUE3000SIX: ", key3000six)
            # ---------------------------------------------------------------------------------------------------------------------- 
            # 
            # ----------------------------------------------------------------------------------------------------------------------                 
            if ack2debug is True:
                print("\n (Step 6B8A/33) OF ZAP3000SIX4XI - TRYING TO SET QUANTITY TO LAYER TO: ", kb3000six [key3000six]  [key2sense] [key2layer] )
                                                                                                                                    #                         
            try:                             
                kb3000six2put [key3000six] [key2sense] [key2layer] [key2qty4layer] = kb3000six [key3000six] [key2sense] [key2layer] [key2qty4layer]
                                                                                                                                    # 
                message2print = "{(Step 6B8B/33) set" + "; qty2layer-of-zap-set-by-zap3000six4xi to: " + str(kb3000six2put [key3000six] [key2sense] [key2layer] [key2qty4layer])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                                  #                 
            except:
                print("\n (Step 6B8C/33) OF ZAP3000SIX4XI *** FAILED TO SET LAYER 3000 TO QUEUE3000SIX: ", key3000six)
            # ---------------------------------------------------------------------------------------------------------------------- 
            # 
            # ----------------------------------------------------------------------------------------------------------------------                 
            if ack2debug is True:
                print("\n (Step 6B9A/33) OF ZAP3000SIX4XI - TRYING TO SET AREA TO LAYER TO: ", kb3000six  [key3000six] [key2sense] [key2layer] )
                                                                                                                                    #                           
            try:                             
                kb3000six2put [key3000six] [key2sense] [key2layer] [key2area4node] = kb3000six [key3000six] [key2sense] [key2layer] [key2area4node]
                                                                                                                                    # AREA2NODE
                message2print = "{(Step 6B9B/33) set" + "; area2node-of-zap-set-by-zap3000six4xi to: " + str(kb3000six2put [key3000six] [key2sense] [key2layer] [key2area4node])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                                  #              
            except:
                print("\n (Step 6B9C/33) OF ZAP3000SIX4XI *** FAILED TO SET LAYER 3000 TO QUEUE3000SIX: ", key3000six)
            # ---------------------------------------------------------------------------------------------------------------------- 
            # 
            # ----------------------------------------------------------------------------------------------------------------------                 
            if ack2debug is True:
                print("\n (Step 6B10A/33) OF ZAP3000SIX4XI - TRYING TO SET IDENTITY TO: ", kb3000six  [key3000six] [key2sense] )  #                
            try:                              
                kb3000six2put [key3000six] [key2sense] [key2identity] = kb3000six[ key3000six] [key2sense] [key2identity]           # IDENTITY
                message2print = "{(Step 6B10B/33) set" + "; identity-of-zap-set-by-zap3000six4xi to: " + str(kb3000six2put [key3000six] [key2sense] [key2identity])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                                  #                 
            except:
                print("\n (Step 6B10C/33) OF ZAP3000SIX4XI *** FAILED TO SET LAYER 3000 TO QUEUE3000SIX: ", key3000six)
            # ---------------------------------------------------------------------------------------------------------------------- 
            # 
            # ----------------------------------------------------------------------------------------------------------------------                 
            if ack2debug is True:
                print("\n (Step 6B11A/33) OF ZAP3000SIX4XI - TRYING TO SET COLOR TO: ", kb3000six [key3000six]  [key2sense] )     #               
            try:                              
                kb3000six2put [key3000six] [key2sense] [key2color] = kb3000six [key3000six]  [key2sense] [key2color]                # COLOR
                message2print = "{(Step 6B4A/33) set" + "; color-of-zap-set-by-zap3000six4xi to: " + str(kb3000six2put [key3000six] [key2sense] [key2color])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                                  #                 
            except:
                print("\n (Step 6B11B/33) OF ZAP3000SIX4XI *** FAILED TO SET LAYER 3000 TO QUEUE3000SIX: ", key3000six)
            # ---------------------------------------------------------------------------------------------------------------------- 
            # 
            # ----------------------------------------------------------------------------------------------------------------------                 
            if ack2debug is True:
                print("\n (Step 6B12A/33) OF ZAP3000SIX4XI - TRYING TO SET FIRE TO: ", kb3000six [key3000six]  [key2sense] )      # 
            try:                             
                kb3000six2put [key3000six] [key2sense] [key2fire] = kb3000six [key3000six]  [key2sense] [key2fire]                  # FIRE 
                message2print = "{STEP 2B12A/33) set" + "; fire-of-zap-set-by-zap3000six4xi to: " + str(kb3000six2put [key3000six] [key2sense] [key2fire])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                                  # 
                        
            except:
                print("\n (Step 6B12B/33) OF ZAP3000SIX4XI *** FAILED TO SET LAYER 3000 TO QUEUE3000SIX: ", key3000six)
            # ---------------------------------------------------------------------------------------------------------------------- 
            # 
            # ----------------------------------------------------------------------------------------------------------------------                 
            if ack2debug is True:
                print("\n (Step 6B13A/33) OF ZAP3000SIX4XI - TRYING TO SET CLUSTER TO: ", kb3000six [key3000six]  [key2sense] )   # 
                                  
            try:                             
                kb3000six2put [key3000six] [key2sense] [key2cluster] = kb3000six [key3000six]  [key2sense] [key2cluster]            # 
                message2print = "{(Step 6B4A/33) set" + "; cluster-set-by-zap3000six4xi to: " + str(kb3000six2put [key3000six] [key2sense] [key2cluster])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                                  #                 
            except:
                print("\n (Step 6B13B/33) OF ZAP3000SIX4XI *** FAILED TO SET LAYER 3000 TO QUEUE3000SIX: ", key3000six)
            # ---------------------------------------------------------------------------------------------------------------------- 
            # 
            # ----------------------------------------------------------------------------------------------------------------------                 
            if ack2debug is True:
                print("\n (Step 6B14A/33) OF ZAP3000SIX4XI - TRYING TO SET RAY: ", kb3000six [key3000six]  [key2sense] )       #             
            try:                             
                kb3000six2put[key3000six] [key2sense] [key2ray] = kb3000six [key3000six]  [key2sense] [key2ray]                     # RAY 
                message2print = "{(Step 6B4A/33) set" + "; ray-of-zap-set-by-zap3000six4xi to: " + str(kb3000six2put[key3000six] [key2sense] [key2ray])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                                  #                
            except:
                print("\n (Step 6B14B/33) OF ZAP3000SIX4XI *** FAILED TO SET LAYER 3000 TO QUEUE3000SIX: ", key3000six)
            # ---------------------------------------------------------------------------------------------------------------------- 
            # 
            # ----------------------------------------------------------------------------------------------------------------------                 
            if ack2debug is True:
                print("\n (Step 6B15A/33) OF ZAP3000SIX4XI - TRYING TO SET DEVIATION TO: ", kb3000six [key3000six]  [key2sense] ) #                 
            try:                             
                kb3000six2put [key3000six] [key2sense] [key2deviation] = kb3000six [key3000six]  [key2sense] [key2deviation]        # DEVIATION
                message2print = "{(Step 6B4A/33) set" + "; deviation-of-zap-set-by-zap3000six4xi to: " + str(kb3000six2put [key3000six] [key2sense] [key2deviation])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                                  #                
            except:
                print("\n (Step 6B15B/33) OF ZAP3000SIX4XI *** FAILED TO SET LAYER 3000 TO QUEUE3000SIX: ", key3000six)
            # ---------------------------------------------------------------------------------------------------------------------- 
            # 
            # ----------------------------------------------------------------------------------------------------------------------                 
            if ack2debug is True:
                print("\n (Step 6B16A/33) OF ZAP3000SIX4XI - TRYING TO SET SUPER TO: ", kb3000six [key3000six]  [key2sense] )     #                
            try:                              
                kb3000six2put[key3000six] [key2sense] [key2super] = kb3000six [key3000six]  [key2sense] [key2super]                 # SUPER
                message2print = "{(Step 6B4A/33) set" + "; super-of-zap-set-by-zap3000six4xi to: " + str(kb3000six2put[key3000six] [key2sense] [key2super])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                                  #                
            except:
                print("\n (Step 6B16B/33) OF ZAP3000SIX4XI *** FAILED TO SET LAYER 3000 TO QUEUE3000SIX: ", key3000six)
            # ---------------------------------------------------------------------------------------------------------------------- 
            # 
            # ----------------------------------------------------------------------------------------------------------------------                 
            if ack2debug is True:
                print("\n (Step 6B17A/33) OF ZAP3000SIX4XI - TRYING TO SET MESSAGE TO: ", kb3000six [key3000six]  [key2sense] )   #               
            try:                             
                kb3000six2put [key3000six] [key2sense] [key2message] = kb3000six [key3000six]  [key2sense] [key2message]            # MESSAGE
                message2print = "{(Step 6B4A/33) set" + "; message-of-zap-set-by-zap3000six4xi to: " + str(kb3000six2put [key3000six] [key2sense] [key2message])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                                  # 
                                 
            except:
                print("\n (Step 6B17C/33) OF ZAP3000SIX4XI - TRYING TO GENERATE DUMMY MESSAGE TO: ", kb3000six [key3000six]  [key2sense])     
                try:
                                                                                                                                    # set default message
                    kb3000six2put [key3000six] [key2sense] [key2message] = "{(Step 6B17C/33) yielded" + "; key2message-yielded-by-zap3000six4xi: " + str(key2message)  + "; time: " + time4xi() + "}"
                    
                    message2print = "{(Step 6B17D/33) set" + "; message-of-zap-set-by-zap3000six4xi to: " + str(kb3000six2put [key3000six] [key2sense] [key2message])  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 

                except:    
                    print("\n (Step 6B17Z/33) OF ZAP3000SIX4XI *** FAILED TO SET MESSAGE AT LAYER 3000: ", key3000six) 
                    
                    
            # ---------------------------------------------------------------------------------------------------------------------- 
            # 
            # ----------------------------------------------------------------------------------------------------------------------                 
            if ack2debug is True:
                print("\n (Step 6B18A/33) OF ZAP3000SIX4XI - TRYING TO SET ACTOR TO: ", kb3000six [key3000six]  [key2sense] )     #                  
            try:                             
                kb3000six2put [key3000six] [key2sense] [key2actor] = kb3000six [key3000six]  [key2sense] [key2actor]                # ACTOR 
                message2print = "{(Step 6B4A/33) set" + "; actor-of-zap-set-by-zap3000six4xi to: " + str(kb3000six2put [key3000six] [key2sense] [key2actor])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                                  #              
            except:
                try:
                    kb3000six2put [key3000six] [key2sense] [key2actor] = 'zap3000six4xi'                          # THIS BECOMES THE ACTOR!
                    message2print = "{(Step 6B18C/33) set" + "; actor-of-zap-set-by-zap3000six4xi to: " + str(kb3000six2put [key3000six] [key2sense] [key2actor])  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                except:                            
                    print("\n (Step 6B18Z/33) OF ZAP3000SIX4XI *** FAILED TO SET ACTOR AT LAYER 3000: ", key3000six)
                    # 

                    
                
                
            # ---------------------------------------------------------------------------------------------------------------------- 
            # 
            # ----------------------------------------------------------------------------------------------------------------------                 
            if ack2debug is True:
                print("\n (Step 6B19A/33) OF ZAP3000SIX4XI - TRYING TO SET TYPE TO: ", kb3000six [key3000six]  [key2sense] )      #               
            try:                             
                kb3000six2put [key3000six] [key2sense] [key2type] = kb3000six [key3000six]  [key2sense] [key2type]                  # TYPE 
                message2print = "{(Step 6B4A/33) set" + "; type-of-zap-set-by-zap3000six4xi to: " + str(kb3000six2put [key3000six] [key2sense] [key2type])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                                  #                
            except:
                try:
                    kb3000six2put [key3000six] [key2sense] [key2type] = 1                                               # function
                    message2print = "{(Step 6B19B/33) set" + "; type-of-zap-set-by-zap3000six4xi to: " + str(kb3000six2put [key3000six] [key2sense] [key2type])  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                                                              # TYPE               
                except:
                    print("\n (Step 6B19Z/33) OF ZAP3000SIX4XI *** FAILED TO SET TYPE AT LAYER 3000: ", key3000six) 
                    # 
                    
            # ---------------------------------------------------------------------------------------------------------------------- 
            # 
            # ----------------------------------------------------------------------------------------------------------------------                 
            if ack2debug is True:
                print("\n (Step 6B20A/33) OF ZAP3000SIX4XI - TRYING TO SET REASON TO: ", kb3000six [key3000six]  )                #                   
            try:                              
                kb3000six2put [key3000six] [key2reason] = kb3000six [key3000six] [key2reason]                                       # REASON
                message2print = "{(Step 6B4A/33) set" + "; reason-of-zap-set-by-zap3000six4xi to: " + str(kb3000six2put [key3000six] [key2reason])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                                  #                 
            except:
                print("\n (Step 6B20B/33) OF ZAP3000SIX4XI *** FAILED TO SET LAYER 3000 TO QUEUE3000SIX: ", key3000six)
            # ---------------------------------------------------------------------------------------------------------------------- 
            # 
            # ----------------------------------------------------------------------------------------------------------------------                 
            if ack2debug is True:
                print("\n (Step 6B21A/33) OF ZAP3000SIX4XI - TRYING TO SET ACE TO: ", kb3000six [key3000six]   )                  #              
            try:                             
                kb3000six2put [key3000six] [key2ace]    = kb3000six [key3000six] [key2ace]                                          # ACE
                message2print = "{(Step 6B4A/33) set" + "; ace-of-zap-set-by-zap3000six4xi to: " + str(kb3000six2put [key3000six] [key2ace])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                                  #                
            except:
                print("\n (Step 6B21B/33) OF ZAP3000SIX4XI *** FAILED TO SET LAYER 3000 TO QUEUE3000SIX: ", key3000six)
                
            # ---------------------------------------------------------------------------------------------------------------------- 
            # 
            # ----------------------------------------------------------------------------------------------------------------------                 
            if ack2debug is True:
                print("\n (Step 6B22A/33) OF ZAP3000SIX4XI - TRYING TO SET MESSAGE TO: ", kb3000six [key3000six]  [key2sense] )     #                 
                                                                                              
            try:
                key2try = kb3000six [key3000six]  [key2sense] [key2actor]
                if ack2debug is True:
                    print("\n (Step 6B22B/33) OF ZAP3000SIX4XI - KEY2TRY: ", key2try) 
                    
                kb3000six2put [key3000six] [key2sense] [key2module] = ModuleKB2Zap4Xi[key2try]                                   
                
                message2print = "{(Step 6B22B/33) set" + "; module-set-by-zap3000six4xi to: " + str(kb3000six2put [key3000six] [key2sense] [key2module])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                                  # 

            except:
                try:
                    kb3000six2put [key3000six] [key2sense] [key2module] = 'zap3000six4xi.py'                      # MODULE IN EXCEPTIONS
                    print("\n (Step 6B22Z1/33) OF ZAP3000SIX4XI ** YIELDED - SET MODULE TO: ", kb3000six2put [key3000six] [key2sense] [key2module])
                    message2print = "{(Step 6B22Z1/33) yielded" + "; module-yielded-by-zap2actor4xi: " + str(kb3000six2put [key3000six] [key2sense] [key2module])  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                          
                except:    
                    print("\n (Step 6B22Z2/33) OF ZAP3000SIX4XI *** FAILED TO SET MODULE AT LAYER 3000 ***")          #  
                    # 
                                                                                                    
 
            # ---------------------------------------------------------------------------------------------------------------------- 
            # 
            # ----------------------------------------------------------------------------------------------------------------------                 
        else:
            pass
            
    # -------------------------------------------------------------------------------------------------
    # (Step 7/33) BUILD LINK LIST AT LAYER 3000 OF DIRECTION SIX 
    # -------------------------------------------------------------------------------------------------               
    if ack2debug is True:
        print("\n (Step 7A/33) OF ZAP3000SIX4XI - TRYING TO BUILD ORDER KNOWLEDGEBASE AT LAYER 3000 OF DIRECTION SIX FROM : ", kb3000six2put)
                                                                                                                                    # 
                                                                                                                                    # 
                                                                                                                                    # 

    try:                             
        keys3000six2put = kb3000six2put.keys()                                                                                      # 
        kb3000link4six = {} 
        order2link = 1
        for key3000six2put in keys3000six2put:
            key3000navigation4six = kb3000six2put [key3000six2put] [key2sense] [key2fire]                                           # input nodes embed fire number 
            kb3000link4six [order2link] = {}
            kb3000link4six [order2link] [key3000six2put] = {}
            kb3000link4six [order2link] [key3000six2put] [1]    = 0                                                                 # order                                                                                                                                 
            kb3000link4six [order2link] [key3000six2put] [2]    = '0x0y0z'                                                          # home                                                           
            kb3000link4six [order2link] [key3000six2put] [3]    = '0x0y0z'                                                          # previous
            kb3000link4six [order2link] [key3000six2put] [4]    = '0x0y0z'                                                          # next
            kb3000link4six [order2link] [key3000six2put] [5]    = '0x0y0z'                                                          # last
            kb3000link4six [order2link] [key3000six2put] [6]    = '0x0y0z'                                                          # first
            kb3000link4six [order2link] [key3000six2put] [7]    = 0                                                                 # fire number
            order2link = order2link + 1
            
        keys3000link4six = kb3000link4six.keys()                                                                                    # keys of input knowledgebase
        if ack2debug is True:
            for key3000link4six in sorted(keys3000link4six):                                                                        # keys of linklist
                if ack2debug is True:
                    print("\n (Step 7B/33) OF ZAP3000SIX4XI KB3000LINK4SIX[", key3000link4six, "]: ", kb3000link4six[key3000link4six])   
                                                                                                                                    # 
                                                                                                                                    # 
    except:
        print("\n (Step 7D/33) OF ZAP3000SIX4XI *** FAILED TO BUILD ORDER KNOWLEDGEBASE AT LAYER 3000 OF DIRECTION SIX: ", kb3000link4six) 
        message2print = "{(Step 7E/33) failed" + "; kb3000link4six-failed-by-zap3000six4xi: " + str(kb3000link4six) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        
    # -------------------------------------------------------------------------------------------------
    # (Step 8A/33) BUILD FIRE LIST
    # -------------------------------------------------------------------------------------------------                       
    if ack2debug is True:
        print("\n (Step 8A1/33) OF ZAP3000SIX4XI - TRYING BUILD FIRE KNOWLEDGEBASE")
    try:
        kb3000fire4six = {}
        keys3000six2put = kb3000six2put.keys()                                                                                      #            
        for key3000six2put in keys3000six2put:
            key3000fire4six = kb3000six2put [key3000six2put] [key2sense] [key2fire]                                                 # 
            kb3000fire4six[key3000fire4six] = key3000six2put                                                                        # 
        if ack2debug is True:
            print("\n (Step 8A2/33) OF ZAP3000SIX4XI - BUILT FIRE KNOWLEDGEBASE: ", kb3000fire4six)                                 
            # 
          
    except:
        print("\n (Step 8A3/33) OF ZAP3000SIX4XI *** FAILED BUILD FIRE KNOWLEDGEBASE")
        message2print = "{(Step 8Z/33) failed" + "; kb3000fire4six-failed-by-zap3000six4xi: " + str(kb3000fire4six) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
       
    # -----------------------------------------------------------------------------------------------------------------------------
    # (Step 8C/33) EMBED FIRES TO LINK LIST  
    # -----------------------------------------------------------------------------------------------------------------------------         
    if ack2debug is True:
        print("\n (Step 8C1/33) OF ZAP3000SIX4XI - TRYING TO EMBED FIRES TO KNOWLEDGEBASE AT LAYER 3000 OF DIRECTION SIX")     

    keys2fire = kb3000fire4six.keys()
    rank2xi = 1
    for key2fire in sorted(keys2fire):                                                                                              #                                                                    
        # -------------------------------------------------------------------------------------------------------------------------
        # 
        # -------------------------------------------------------------------------------------------------------------------------                                                                                     
        if ack2debug is True:
            print("\n (Step 8C5/33) OF ZAP3000SIX4XI - KEY2FIRE: ", key2fire)                                                       # 

        coordinate2rank = kb3000fire4six[key2fire]                                                                                  #                                                             
        
        
        keys2link = kb3000link4six.keys()                                                                                           
        for keylink in keys2link:                                                                                                   # 
            try:
                if kb3000link4six [keylink] [coordinate2rank]  [7] == 0:            
                    kb3000link4six [keylink] [coordinate2rank] [7] = key2fire                                                       # 
                if kb3000link4six [keylink] [coordinate2rank]  [1] == 0:
                    kb3000link4six [keylink] [coordinate2rank] [1] = rank2xi                                                        # 
                    rank2xi = rank2xi + 1
                if ack2debug is True:    
                    print("\n (Step 8D10/33) OF ZAP3000SIX4XI - MATCH: UPDATED KB3000LINK4SIX: ", kb3000link4six)                   # 
                                                                                                                                    # 
                                                                                                                                    # 
            except:
                pass               

    
    # ---------------------------------------------------------------------------------------------------------------------
    # (Step 8D/33) PRINT LINKING LIST WITH SIX FIRE EMBEDDED  
    # ---------------------------------------------------------------------------------------------------------------------                    
    if ack2debug is True:
        print("\n (Step 8D1/33) OF ZAP3000SIX4XI - TRYING TO PRINT LINKING LIST WITH SIX FIRE EMBEDDED ")   
        try: 
            print("\n (Step 8C7/33) OF ZAP3000SIX4XI - KB3000LINK4SIX: ", kb3000link4six)                                           # 
        except:                                                                                                                         
            print("\n (Step 8Z4/33) OF ZAP3000SIX4XI *** FAILED TO PRINT LINKING LIST WITH SIX FIRE EMBEDDED") 
                        
            
    # -------------------------------------------------------------------------------------------------
    # (Step 8E/33) EMBED FIRES TO KNOWLEDGEBASE AT LAYER 3000 OF DIRECTION SIX
    # -------------------------------------------------------------------------------------------------         
    if ack2debug is True:
        print("\n (Step 8D1/33) OF ZAP3000SIX4XI - TRYING TO EMBED FIRES TO KNOWLEDGEBASE OF LAYER 3000 OF DIRECTION SIX")      
                                                                                                                           
    keys3000six2put = kb3000six2put.keys()
                                               
    # -------------------------------------------------------------------------------------------------
    # THE LATEST VALUES TO BE CHANGED DURING WALKTROUGHS
    # -------------------------------------------------------------------------------------------------     
    origin2link         = '0x0y0z'                                                                                                  # link to origin
    home2link           = '-0x-0y-0z'                                                                                               # unresolved-home  
    previous2link       = '-0x-0y-0z'                                                                                               # unresolved-previous 
    next2link           = '+0x+0y+0z'                                                                                               # unresolved-next 
    first2link          = '+0x+0y+0z'                                                                                               # unresolved-first 
    last2link           = '+0x+0y+0z'                                                                                               # unresolved-last 
    max2last            = 0                                                                                                         # maximun order number for the last node - default 0

                                                                                                             
    # -------------------------------------------------------------------------------------------------
    # (Step 8E1/33) THE FIRST WALKTROUGH TO EMBED FIRES TO KNOWLEDGEBASE OF LAYER 3000 OF DIRECTION SIX
    # -------------------------------------------------------------------------------------------------         
    if ack2debug is True:
        print("\n (Step 8E1a/33) OF ZAP3000SIX4XI - TRYING THE FIRST WALKTROUGH TO EMBED FIRES TO KNOWLEDGEBASE OF LAYER 3000 OF DIRECTION SIX")               
    for key3000six2put in keys3000six2put:
        # -------------------------------------------------------------------------------------------------
        # KNOWLEDGEBASE AT LAYER 3000 OF DIRECTION SIX
        # -------------------------------------------------------------------------------------------------                                                                                      
        if ack2debug is True:
            print("\n (Step 8E1b/33) OF ZAP3000SIX4XI KB3000LINK4SIX - KEY3000SIX2PUT: ", key3000six2put)                           # 
        keys2link = kb3000link4six.keys()
        for key2link in keys2link:
            if ack2debug is True:
                print("\n (Step 8E1c/33) OF ZAP3000SIX4XI KB3000LINK4SIX - KEY2LINK: ", key2link)                                   # 
            # ---------------------------------------------------------------------------------------------
            # LINK LIST AT LAYER 3000 OF DIRECTION SIX
            # ---------------------------------------------------------------------------------------------          
            try:
                kb2check = kb3000link4six[key2link]
                keys2check = kb2check.keys()
                for key2check in keys2check:
                    # ---------------------------------------------------------------------------------------------
                    # NODE(S) OF LINK LIST AT LAYER 3000 OF DIRECTION SIX
                    # -------------------------------------------------------------------------------------------                   
                    if key2check == key3000six2put: 
                        if ack2debug is True:
                            print("\n (Step 8E1d/33) OF ZAP3000SIX4XI KB3000LINK4SIX - MATCH WITH KEY3000SIX2PUT: ", key3000six2put) 
                                                                                                                                    #                                                                         
                        if kb3000link4six[key2link] [key3000six2put] [1] == 1:                                                      # the first nsix in fire order at this layer to required direction
                            if ack2debug is True:
                                print("\n (Step 8E1e/33) OF ZAP3000SIX4XI KB3000LINK4SIX - THE FIRT NODE: ", key3000six2put)        # 
                                  
                            # ---------------------------------------------------------------------------------
                            # SET LINKS OF THE FIRST NODE
                            # ---------------------------------------------------------------------------------                                                                              
                            first2link        = key3000six2put                                                                      # first-link points to this node                        
                            if ack2debug is True:
                                print("\n (Step 8E1f/33) OF ZAP3000SIX4XI KB3000LINK4SIX - SET FIRST2LINK TO: ", first2link)            
                                
                            # ---------------------------------------------------------------------------------
                            # EMBED LINKS OF THE FIRST NODE
                            # ---------------------------------------------------------------------------------                                                    
                            kb3000six2put [key3000six2put] [key2sense] [key2home]     = home2link                                   # 
                                                                     
                                                     
                            # ---------------------------------------------------------------------------------
                            # UPDATE AFTER THE FIRST NODE
                            # --------------------------------------------------------------------------------- 
                            if kb3000link4six[key2link] [key3000six2put] [1] > max2last:
                                last2link    = key3000six2put                                                                       # 
                                max2last     = kb3000link4six[key2link] [key3000six2put] [1]                                        # 
                        else:
                            if ack2debug is True:
                                print("\n (Step 8E1g/33) OF ZAP3000SIX4XI KB3000LINK4SIX - THE ", kb3000link4six[key2link] [key3000six2put] [1], ". NODE: ", key3000six2put)
                                                                                                                                    # 
                                                                                                                                    #         
                            # ---------------------------------------------------------------------------------
                            # SET LINKS OF TYPICAL NODE
                            # ---------------------------------------------------------------------------------
                                                                                                                                    # 
                            
                            # ---------------------------------------------------------------------------------
                            # EMBED LINKS OF TYPICAL NODE
                            # ---------------------------------------------------------------------------------                                                    
                            kb3000six2put [key3000six2put] [key2sense] [key2home]     = home2link                                   # only link to be updated for output!

                            # ---------------------------------------------------------------------------------
                            # UPDATE AFTER TYPICAL NODE 
                            # --------------------------------------------------------------------------------- 
                            if kb3000link4six[key2link] [key3000six2put] [1] > max2last:
                                last2link    = key3000six2put                                                                       # 
                                max2last     = kb3000link4six[key2link] [key3000six2put] [1]                                        #                   
                                                      
                           
                    else:
                        if ack2debug is True:
                            print("\n (Step 8E1h/33) OF ZAP3000SIX4XI KEY3000SIX2PUT - PASSED KEY2LINK: ", key2link, " WITH NODE: ", key3000six2put)              
            except:
                pass                                                                                                                # nodes with different links
    # -------------------------------------------------------------------------------------------------
    # 
    # -------------------------------------------------------------------------------------------------                                  
    if ack2debug is True:
        # key2home        = 14 [OK]
        print("\n (Step 8E1i/33) OF ZAP3000SIX4XI - LATEST FIRST: ", first2link)                                                    # 
        print("\n (Step 8E1j/33) OF ZAP3000SIX4XI - LATEST LAST: ", last2link)                                                      #                            
        print("\n (Step 8E1k/33) OF ZAP3000SIX4XI - LATEST MAX ORDER: ",max2last)                                                   #           
                
    # -------------------------------------------------------------------------------------------------
    # (Step 9/33) THE SECOND WALKTROUGH TO EMBED THE FIRST AND LAST LINK TO KNOWLEDGEBASE
    # -------------------------------------------------------------------------------------------------         
    if ack2debug is True:
        print("\n (Step 9A/33) OF ZAP3000SIX4XI - TRYING TO EMBED THE FIRST AND LAST LINK  TO KNOWLEDGEBASE OF LAYER 3000 OF DIRECTION SIX")
    try:      
        for key3000six2put in keys3000six2put:
            # ---------------------------------------------------------------------------------------------
            # KNOWLEDGEBASE AT LAYER 3000 OF DIRECTION SIX
            # ---------------------------------------------------------------------------------------------    
            kb3000six2put [key3000six2put] [key2sense] [key2first] = first2link
            kb3000six2put [key3000six2put] [key2sense] [key2last]  = last2link
        # ------------------------------------------------------------------------------------------------- 
        #
        # -------------------------------------------------------------------------------------------------             
        if ack2debug is True:    
            print("\n (Step 9B/33) OF ZAP3000SIX4XI - UPDATED THE FIRST AND LAST LINK TO KNOWLEDGEBASE OF LAYER 3000 OF DIRECTION SIX")
            print("\n (Step 9C/33) OF ZAP3000SIX4XI - KB3000SIX2PUT: ", kb3000six2put)
            # 
 
    except:        
        print("\n (Step 9Z/33) OF ZAP3000SIX4XI *** FAILED TO EMBED THE FIRST AND LAST LINK  TO KNOWLEDGEBASE OF LAYER 3000 OF DIRECTION SIX")
        message2print = "{(Step 9Z/33) failed" + "; kb3000six2put-failed-by-zap3000six4xi: " + str(kb3000six2put) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                  
    # -------------------------------------------------------------------------------------------------
    # (Step 10/33) THE THIRD WALKTROUGH TO EMBED PREVIOUS LINKS TO KNOWLEDGEBASE
    # -------------------------------------------------------------------------------------------------         
    if ack2debug is True:
        print("\n (Step 10A/33) OF ZAP3000SIX4XI - TRYING TO EMBED PREVIOUS LINKS TO KNOWLEDGEBASE OF LAYER 3000 OF DIRECTION SIX")
    try:    
        # 
        
        keys2link = kb3000link4six.keys()
        for key2link in keys2link:
            if ack2debug is True:
                print("\n (Step 10B/33) OF ZAP3000SIX4XI KB3000LINK4SIX - KEY2LINK: ", key2link)                                    # 
            # ---------------------------------------------------------------------------------------------------
            # LINK LIST AT LAYER 3000 OF DIRECTION SIX
            # ---------------------------------------------------------------------------------------------------          
            try:
                kb2check = kb3000link4six[key2link]                                                                                 # 
                keys2check = kb2check.keys()                                                                                        # 
                for key2check in keys2check: 
                                                                                                                                    #  
                    # -------------------------------------------------------------------------------------------
                    # PREVIOUS LINK
                    # -------------------------------------------------------------------------------------------                                                                                                                                             
                    if kb2check [key2check] [1] == 1:                                                                               # 1 = the first node
                        # ---------------------------------------------------------------------------------------
                        # FIRST NODE - UNRESOLVED-PREVIOUS FOR THE MAIN TO SOLVE FROM PREVIOUS LAYER LAST 
                        # ---------------------------------------------------------------------------------------
                        kb3000six2put [key2check] [key2sense] [key2previous] = previous2link                                        # 
                        if ack2debug is True:
                            print("\n (Step 10C/33) OF ZAP3000SIX4XI - PREVIOUS SET TO: ", previous2link, "  FOR THE FIRST NODE: ", key2check)
                                                                                                                                    # 
                    else:
                        # ---------------------------------------------------------------------------------------
                        # PREVIOUS TO OTHERS
                        # ---------------------------------------------------------------------------------------                 
                        order2previous = kb2check [key2check] [1] - 1                                                               # 
                        if ack2debug is True:
                            print("\n (Step 10D/33) OF ZAP3000SIX4XI ORDER OF PREVIOUS TO HUNT: ", order2previous, "  FOR NODE: ", key2check)
                                                                                                                                    # 
                                                                                                                                    # 
                        keys2hunt1 = kb3000link4six.keys()                                  
                        for key2hunt1 in keys2hunt1:                                                                                # 
                            # -----------------------------------------------------------------------------------
                            # HUNT PREVIOUS
                            # -----------------------------------------------------------------------------------
                            kb2check4hunt1 = kb3000link4six[key2hunt1]                                                              # 
                            keys2check4hunt1 = kb2check4hunt1.keys()                                                                # 
                            for key2check4hunt1 in keys2check4hunt1:
                                if kb2check4hunt1 [key2check4hunt1] [1] == order2previous:                                          # 
                                    kb3000six2put [key2check] [key2sense] [key2previous] = key2check4hunt1                          # 
                                    if ack2debug is True:
                                        print("\n (Step 10E/33) OF ZAP3000SIX4XI - PREVIOUS SET TO: ", key2check4hunt1, "  FOR NODE: ", key2check)
                                                                                                                                    # 
                                                                                                                                    # 
                            else:
                                pass

            except:        
                pass                    
        # ------------------------------------------------------------------------------------------------- 
        #
        # ------------------------------------------------------------------------------------------------- 
        # input link list:
        #                                          
        if ack2debug is True:    
            print("\n (Step 10B/33) OF ZAP3000SIX4XI - UPDATED PREVIOUS LINKS TO KNOWLEDGEBASE OF LAYER 3000 OF DIRECTION SIX")
            print("\n (Step 10C/33) OF ZAP3000SIX4XI - KB3000SIX2PUT: ", kb3000six2put)
            # 
            
    except:        
        print("\n (Step 10Z2/33) OF ZAP3000SIX4XI *** FAILED TO EMBED PREVIOUS LINKS TO KNOWLEDGEBASE OF LAYER 3000 OF DIRECTION SIX")
        message2print = "{(Step 10Z/33) failed" + "; kb3000six2put-failed-by-zap3000six4xi: " + str(kb3000six2put) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
    # -------------------------------------------------------------------------------------------------
    # (Step 11/33) THE THIRD WALKTROUGH TO EMBED NEXT LINKS TO KNOWLEDGEBASE
    # -------------------------------------------------------------------------------------------------         
    if ack2debug is True:
        print("\n (Step 11A/33) OF ZAP3000SIX4XI - TRYING TO EMBED NEXT LINKS TO KNOWLEDGEBASE OF LAYER 3000 OF DIRECTION SIX")
    try:
        # input link list:    
        # 
        
        keys2link = kb3000link4six.keys()
        for key2link in keys2link:
            if ack2debug is True:
                print("\n (Step 11B/33) OF ZAP3000SIX4XI KB3000LINK4SIX - KEY2LINK: ", key2link)                                    # 
            # ---------------------------------------------------------------------------------------------------
            # LINK LIST AT LAYER 3000 OF DIRECTION SIX
            # ---------------------------------------------------------------------------------------------------          
            try:
                kb2check = kb3000link4six[key2link]                                                                                 # 
                keys2check = kb2check.keys()                                                                                        # 

                for key2check in keys2check:                 
                    # -------------------------------------------------------------------------------------------
                    # NEXT LINK
                    # -------------------------------------------------------------------------------------------                                 
                    if kb2check [key2check] [1] == max2last:
                        # ---------------------------------------------------------------------------------------
                        # LAST NODE - UNRESOLVED-NEXT FOR THE MAIN TO SOLVE FROM NEXT LAYER FIRST
                        # --------------------------------------------------------------------------------------- 
                        kb3000six2put [key2check] [key2sense] [key2next] = next2link                                                # 
                        if ack2debug is True:
                            print("\n (Step 11F/33) OF ZAP3000SIX4XI - NEXT SET TO: ", next2link, "  FOR LAST NODE: ", key2check)
                                                                                                                                    #                      
                    else:
                        # ---------------------------------------------------------------------------------------
                        # NEXT TO OTHERS
                        # ---------------------------------------------------------------------------------------                     
                        order2next = kb2check [key2check] [1] + 1                                                                   # 
                        if ack2debug is True:
                            print("\n (Step 11E/33) OF ZAP3000SIX4XI ORDER OF NEXT TO HUNT: ", order2next, "  FOR NODE: ", key2check)
                                                                                                                                    # 
                                                                                                                                    #                    
                        keys2hunt2 = kb3000link4six.keys()                         
                        for key2hunt2 in keys2hunt2:
                            # -----------------------------------------------------------------------------------
                            # HUNT NEXT
                            # -----------------------------------------------------------------------------------
                            kb2check4hunt2 = kb3000link4six[key2hunt2] 
                            keys2check4hunt2 = kb2check4hunt2.keys()
                            for key2check4hunt2 in keys2check4hunt2: 
                                if kb2check4hunt2 [key2check4hunt2] [1] == order2next:                                                
                                    kb3000six2put [key2check] [key2sense] [key2next] = key2check4hunt2                              # 
                                    if ack2debug is True:
                                        print("\n (Step 11G/33) OF ZAP3000SIX4XI - NEXT SET TO: ", key2check4hunt2, "  FOR NODE: ", key2check)
                                                                                                                                    #            
                                else:
                                    pass
            except:        
                pass                    
        # ------------------------------------------------------------------------------------------------- 
        #
        # ------------------------------------------------------------------------------------------------- 
        # order numbers:
        #       
        # keys:
        # key2direction   = 1,  key2distance    = 2,  key2layer       = 3   {key2area4layer  = 1, key2color4layer = 2, key2qty4layer   = 3, key2area4node   = 4}
        # key2identity    = 4,  key2color       = 5,  key2fire        = 6,  key2cluster     = 7,  key2ray         = 8, key2deviation   = 9, key2super       = 10
        # key2message     = 11, key2actor       = 12, key2type        = 13, key2home        = 14
        # key2previous    = 15, key2next        = 16, key2last        = 17, key2first       = 18, key2module      = 19
                                                  
        if ack2debug is True:    
            print("\n (Step 11B/33) OF ZAP3000SIX4XI - UPDATED NEXT LINKS TO KNOWLEDGEBASE OF LAYER 3000 OF DIRECTION SIX")
            print("\n (Step 11C/33) OF ZAP3000SIX4XI - KB3000SIX2PUT: ", kb3000six2put)
            # 

    except:        
        print("\n (Step 10Z2/33) OF ZAP3000SIX4XI *** FAILED TO EMBED NEXT AND PREVIOUS LINKS TO KNOWLEDGEBASE OF LAYER 3000 OF DIRECTION SIX")
        message2print = "{(Step 10Z2/33) failed" + "; kb3000six2put-failed-by-zap3000six4xi: " + str(kb3000six2put) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                                                
    # -------------------------------------------------------------------------------------------------
    # (Step 21/33) PUT KNOWLEDGEBASE FOR LAYER 3000 OF DIRECTION SIX 
    # -------------------------------------------------------------------------------------------------                   
    try:
        queue3000six.put(kb3000six2put)
        ack3000put4six = True  
        if ack2debug is True:                                                                                                       # 
            print("\n (Step 21A/33) OF ZAP3000SIX4XI - SUCCEEDED TO PUT LAYER 3000 TO QUEUE: ", queue3000six, " KB2PUT: ", kb3000six2put)
            # 
       
        # minimize trace after module test:
        message2print = "{(Step 21B/33) put" + "; kb3000six2put-put-by-zap3000six4xi: " + str(kb3000six2put)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                                          #
                    
    except:                                                                                                                
        print("\n (Step 21C/33) OF ZAP3000SIX4XI *** FAILED TO SET LAYER 3000 TO QUEUE3000SIX: ", queue3000six, " KB2PUT: ", kb3000six2put)                                                                                                         
        message2print = "{(Step 21C/33) failed" + "; queue3000six-put-failed-by-zap3000six4xi: " + str(queue3000six.put) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
           
    # -------------------------------------------------------------------------------------------------
    # (Step 20/33) MESSAGE ABOUT PUT OF KNOWLEDGEBASE FOR LAYER 3000 OF DIRECTION SIX 
    # -------------------------------------------------------------------------------------------------   
    if ack2debug is True:
        print("\n (Step 20A/33) OF ZAP3000SIX4XI - TRYING TO INFORM PUT OPERATIONS VIA PIPE'S PUBLISHER_CHANNEL3000SIX: ", publisher_channel3000six)
                                                                                                                                    # 
    if ack3000put4six is True:
        message2send4six = "<ack3000put4six4xi>layer3000six</ack3000put4six4xi>"
    else:
        message2send4six = "<nack3000put4six4xi>layer3000six</nack3000put4six4xi>" 
    message2print = "{(Step 20B/33) set" + "; message2send4six-set-by-zap3000six4xi to: " + str(message2send4six)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                                              #       
     
                
    # -------------------------------------------------------------------------------------------------
    # (Step 21/33) SEND MESSAGE ABOUT PUT OF KNOWLEDGEBASE FOR LAYER 3000 OF DIRECTION SIX 
    # -------------------------------------------------------------------------------------------------                        
    try:                
        # ---------------------------------------------------------------------------------------------
        # INFORM PUT OPERATIONS VIA PIPE'S PUBLISHER CHANNEL 
        # ---------------------------------------------------------------------------------------------
        publisher_channel3000six.send(message2send4six)
        if ack2debug is True:     
            print("(Step 21A/33) OF ZAP3000SIX4XI -  SENT MESSAGE2SEND4SIX: ", message2send4six, " TO PUBLISHER_CHANNEL2SIX: ", publisher_channel3000six, " - TIME: ", time4xi()) 
                                                                                                                                    # 
        message2print = "{(Step 21B/33) sent" + "; message2send4six-sent-by-zap3000six4xi: " + str(message2send4six)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                                          #  
    except:
        print("(Step 21C/33) OF ZAP3000SIX4XI *** FAILED TO SEND MESSAGE2SEND4SIX: ", message2send4six, " TO PUBLISHER_CHANNEL2SIX: ", publisher_channel3000six, " - TIME: ", time4xi()) 
        message2print = "{(Step 21C/33) failed" + "; message2send4six-failed-by-zap3000six4xi: " + str(message2send4six) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                                          # 
        
         
    # -------------------------------------------------------------------------------------------------
    # (Step 22/33) RETURN QUEUE OF LAYER 3000 OF DIRECTION SIX 
    # -------------------------------------------------------------------------------------------------               
    if ack2debug is True:        
        print("\n (Step 22A/33) OF ZAP3000SIX4XI - RETURNING QUEUE OBJECT QUEUE3000SIX: ", queue3000six, " - TIME: ", time4xi()) 
         
    message2print = "{(Step 22B/33) returning" + "; queue3000six-returning-by-zap3000six4xi: " + str(queue3000six) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                              # 
        
    return queue3000six    
        
#</function4xi>


#</program4xi>
# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   zap2actor4xi.py
#   test_zap2actor4xi.py
#   test_zap2actor4xi.txt
#   sense-test-zap2actor4xi.xml
#   reason-test-zap2actor4xi.xml
#   ace-test-zap2actor4xi4xi.xml
#   scope-sense*4xi.xml
#"
#>

# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   references = "Legal notice at www.dragonxi.com; www.yourdragonxi.com; www.yourdragonxi.org; www.dragon2xi.com/; www.dragon4xi.com/; www.dragon8xi.com/ " 
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>
 
#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of zap3000six4xi.py                           
# ------------------------------------------------------------------------------------------------- 
#"
#/>
